@@ -81,7 +81,7 @@ RE_PATTERN_RESIZE = re.compile(r'\x1b\[48;(?P<height_chars>\d+);(?P<width_chars>
                                r';(?P<height_pixels>\d+);(?P<width_pixels>\d+)t')
 
 # DEC event pattern container
-DECEventPattern = functools.namedtuple("DEC_EVENT_PATTERN", ["mode", "pattern"])
+DECEventPattern = namedtuple("DECEventPattern", ["mode", "pattern"])
 
 # DEC event patterns - compiled regexes with metadata of the 'mode' that
 # triggered it, this prevents searching for bracketed paste or mouse modes
@@ -292,7 +292,6 @@ class Keystroke(str):
 
         Returns name like 'KEY_CTRL_ALT_A', 'KEY_ALT_SHIFT_5', 'KEY_CTRL_J_RELEASED', etc.
         """
-        # pylint: disable=too-many-return-statements
         if self._mode != DecPrivateMode.SpecialInternalKitty:
             return None
 
@@ -887,7 +886,6 @@ class Keystroke(str):
         Handles Alt+printable, Alt-only special keys, and Ctrl+Alt sequences. Returns the base
         character or empty string for application keys.
         """
-        # pylint: disable=too-many-return-statements
         if not self._is_escape_sequence():
             return None
 
@@ -1081,7 +1079,6 @@ class Keystroke(str):
     def _mode_values(self) -> Optional[typing.Union[BracketedPasteEvent,
                                                     MouseSGREvent, MouseLegacyEvent, FocusEvent,
                                                     ResizeEvent]]:
-        # pylint: disable=too-many-return-statements
         """
         Return structured data for DEC private mode events (private API).
 
@@ -1186,7 +1183,7 @@ def get_keyboard_codes() -> Dict[int, str]:
     # Merge in homemade KEY_TAB, KEY_KP_*, KEY_MENU added to our module space
     # Exclude *_PUA constants since they're internal implementation details
     # that will be remapped via KITTY_PUA_KEYCODE_OVERRIDE_MIXIN.
-    # Make sure to copy globals() since other threads might create or 
+    # Make sure to copy globals() since other threads might create or
     # destroy globals while we iterate.
     keycodes.update((k, v) for k, v in globals().copy().items()
                     if k.startswith('KEY_') and not k.endswith('_PUA'))
@@ -1412,14 +1409,13 @@ def _read_until(term: 'Terminal',
     must ensure any such keyboard data is well-received by the next call to
     term.inkey() without delay.
     """
+    # Maximum buffer size to prevent runaway condition -- one such example is automatic terminal
+    # responses that get echoed back indefinitely in an accidental LINEMODE telnet server. 64KB is
+    # far more than any legitimate automatic terminal response could be.
+    max_buffer_size = 65536
+
     stime = time.time()
     match, buf = None, ''
-
-    # first, buffer all pending data. pexpect library provides a
-    # 'searchwindowsize' attribute that limits this memory region.  We're not
-    # concerned about OOM conditions: only (human) keyboard input and terminal
-    # response sequences are expected.
-
     while True:  # pragma: no branch
         # block as long as necessary to ensure at least one character is
         # received on input or remaining timeout has elapsed.
@@ -1430,6 +1426,9 @@ def _read_until(term: 'Terminal',
         # for short timeout periods.
         while ucs:
             buf += ucs
+            # Check buffer size limit to catch echo loops early
+            if len(buf) > max_buffer_size:
+                break
             ucs = term.inkey(timeout=0, esc_delay=0)
 
         match = re.search(pattern=pattern, string=buf)
@@ -1441,6 +1440,10 @@ def _read_until(term: 'Terminal',
             # timeout
             break
 
+        if len(buf) > max_buffer_size:
+            # buffer overflow - likely an echo loop or misbehaving terminal
+            break
+
     return match, buf
 
 
@@ -2058,7 +2061,7 @@ KITTY_PUA_KEYCODE_OVERRIDE_MIXIN = (
 #: escape sequences.  This is too long for modern applications, so we set it to
 #: 350ms, or 0.35 seconds. It is still a bit conservative, for remote telnet or
 #: ssh servers, for example.
-DEFAULT_ESCDELAY = 0.35  # pylint: disable=invalid-name
+DEFAULT_ESCDELAY = 0.35
 
 
 def _reinit_escdelay() -> None:
