@@ -1,35 +1,36 @@
-# -*- coding: utf-8 -*-
 """Module providing 'sequence awareness'."""
+from __future__ import annotations
+
 # std imports
 import re
-import sys
-import math
-import textwrap
-from typing import TYPE_CHECKING
+from typing import TYPE_CHECKING, Tuple, Pattern, Iterator, Optional, SupportsIndex
 
 # 3rd party
-from wcwidth import wcwidth
+from wcwidth import SequenceTextWrapper  # noqa: F401  # re-exported for API compatibility
+from wcwidth import clip as wcwidth_clip
+from wcwidth import ljust as wcwidth_ljust
+from wcwidth import rjust as wcwidth_rjust
+from wcwidth import width as wcwidth_width
+from wcwidth import center as wcwidth_center
 
 # local
 from blessed._capabilities import CAPABILITIES_CAUSE_MOVEMENT, CAPABILITIES_HORIZONTAL_DISTANCE
 
 if TYPE_CHECKING:  # pragma: no cover
+    # local
     from blessed.terminal import Terminal
 
-# std imports
-from typing import List, Type, Tuple, Pattern, TypeVar, Iterator, Optional
-
-# SupportsIndex was added in Python 3.8
-if sys.version_info >= (3, 8):
-    # std imports
-    from typing import SupportsIndex
-else:
-    SupportsIndex = int
-
-_T = TypeVar("_T")
-
 __all__ = ('Sequence', 'SequenceTextWrapper', 'iter_parse', 'measure_length')
 
+# Translation table to remove C0 and C1 control characters.
+# These cause wcswidth() to return -1, but should be ignored for width calculation
+# since terminal sequences are already stripped before measurement.
+_CONTROL_CHAR_TABLE = str.maketrans('', '', (
+    ''.join(chr(c) for c in range(0x00, 0x20)) +  # C0: U+0000-U+001F
+    '\x7f' +                                      # DEL
+    ''.join(chr(c) for c in range(0x80, 0xA0))    # C1: U+0080-U+009F
+))
+
 
 class Termcap():
     """Terminal capability of given variable name and pattern."""
@@ -144,121 +145,6 @@ class Termcap():
         return cls(name, re.sub(pattern, lambda x: _numeric_regex, _outp), attribute, nparams)
 
 
-class SequenceTextWrapper(textwrap.TextWrapper):
-    """Docstring overridden."""
-
-    def __init__(self, width: int, term: 'Terminal', **kwargs: object) -> None:
-        """
-        Class initializer.
-
-        This class supports the :meth:`~.Terminal.wrap` method.
-        """
-        self.term = term
-        textwrap.TextWrapper.__init__(self, width, **kwargs)
-
-    def _wrap_chunks(self, chunks):    # type: ignore[no-untyped-def]
-        """
-        Sequence-aware variant of :meth:`textwrap.TextWrapper._wrap_chunks`.
-
-        :raises ValueError: ``self.width`` is not a positive integer
-        :rtype: list
-        :returns: text chunks adjusted for width
-
-        This simply ensures that word boundaries are not broken mid-sequence, as standard python
-        textwrap would incorrectly determine the length of a string containing sequences, and may
-        also break consider sequences part of a "word" that may be broken by hyphen (``-``), where
-        this implementation corrects both.
-        """
-        lines: List[str] = []
-        if self.width <= 0 or not isinstance(self.width, int):
-            raise ValueError(
-                f"invalid width {self.width!r}({type(self.width)!r}) (must be integer > 0)"
-            )
-
-        term = self.term
-        drop_whitespace = not hasattr(self, 'drop_whitespace'
-                                      ) or self.drop_whitespace
-        chunks.reverse()
-        while chunks:
-            cur_line: List[str] = []
-            cur_len = 0
-            indent = self.subsequent_indent if lines else self.initial_indent
-            width = self.width - len(indent)
-            if drop_whitespace and lines and not Sequence(chunks[-1], term).strip():
-                del chunks[-1]
-            while chunks:
-                chunk_len = Sequence(chunks[-1], term).length()
-                if cur_len + chunk_len > width:
-                    if chunk_len > width:
-                        self._handle_long_word(chunks, cur_line, cur_len, width)
-                    break
-                cur_line.append(chunks.pop())
-                cur_len += chunk_len
-            if drop_whitespace and (cur_line and not Sequence(cur_line[-1], term).strip()):
-                del cur_line[-1]
-            if cur_line:
-                lines.append(f'{indent}{"".join(cur_line)}')
-        return lines
-
-    def _handle_long_word(self,  # type: ignore[no-untyped-def]
-                          reversed_chunks, cur_line, cur_len, width):
-        """
-        Sequence-aware :meth:`textwrap.TextWrapper._handle_long_word`.
-
-        This method ensures that word boundaries are not broken mid-sequence, as
-        standard python textwrap would incorrectly determine the length of a
-        string containing sequences and wide characters it would also break
-        these "words" that would be broken by hyphen (``-``), this
-        implementation corrects both.
-
-        This is done by mutating the passed arguments, removing items from
-        'reversed_chunks' and appending them to 'cur_line'.
-
-        However, some characters (east-asian, emoji, etc.) cannot be split any
-        less than 2 cells, so in the case of a width of 1, we have no choice
-        but to allow those characters to flow outside of the given cell.
-        """
-        # Figure out when indent is larger than the specified width, and make
-        # sure at least one character is stripped off on every pass
-        space_left = 1 if width < 1 else width - cur_len
-        # If we're allowed to break long words, then do so: put as much
-        # of the next chunk onto the current line as will fit.
-
-        if self.break_long_words:
-            term = self.term
-            chunk = reversed_chunks[-1]
-            idx = nxt = seq_length = 0
-            for text, _ in iter_parse(term, chunk):
-                nxt += len(text)
-                seq_length += Sequence(text, term).length()
-                if seq_length > space_left:
-                    if cur_len == 0 and width == 1 and nxt == 1 and seq_length == 2:
-                        # Emoji etc. cannot be split under 2 cells, so in the
-                        # case of a width of 1, we have no choice but to allow
-                        # those characters to flow outside of the given cell.
-                        pass
-                    else:
-                        break
-                idx = nxt
-            cur_line.append(chunk[:idx])
-            reversed_chunks[-1] = chunk[idx:]
-
-        # Otherwise, we have to preserve the long word intact.  Only add
-        # it to the current line if there's nothing already there --
-        # that minimizes how much we violate the width constraint.
-        elif not cur_line:
-            cur_line.append(reversed_chunks.pop())
-
-        # If we're not allowed to break long words, and there's already
-        # text on the current line, do nothing.  Next time through the
-        # main loop of _wrap_chunks(), we'll wind up here again, but
-        # cur_len will be zero, so the next line will be entirely
-        # devoted to the long word that we can't handle right now.
-
-
-SequenceTextWrapper.__doc__ = textwrap.TextWrapper.__doc__
-
-
 class Sequence(str):
     """
     A "sequence-aware" version of the base :class:`str` class.
@@ -266,9 +152,12 @@ class Sequence(str):
     This unicode-derived class understands the effect of escape sequences
     of printable length, allowing a properly implemented :meth:`rjust`,
     :meth:`ljust`, :meth:`center`, and :meth:`length`.
+
+    .. note:: that other than :meth:`Sequence.padd`, this is just a thin layer
+        over :mod:`wcwidth`, kept by name for API compatibility.
     """
 
-    def __new__(cls: Type[_T], sequence_text: str, term: 'Terminal') -> _T:
+    def __new__(cls, sequence_text: str, term: 'Terminal') -> Sequence:
         """
         Class constructor.
 
@@ -283,88 +172,68 @@ class Sequence(str):
         """
         Return string containing sequences, left-adjusted.
 
-        :arg int width: Total width given to left-adjust ``text``.  If
-            unspecified, the width of the attached terminal is used (default).
+        :arg int width: Total width given to left-adjust ``text``.
         :arg str fillchar: String for padding right-of ``text``.
         :returns: String of ``text``, left-aligned by ``width``.
         :rtype: str
         """
-        rightside = fillchar * int(
-            (max(0.0, float(width.__index__() - self.length()))) / float(len(fillchar)))
-        return ''.join((self, rightside))
+        return wcwidth_ljust(self, width.__index__(), fillchar, control_codes='ignore')
 
     def rjust(self, width: SupportsIndex, fillchar: str = ' ') -> str:
         """
         Return string containing sequences, right-adjusted.
 
-        :arg int width: Total width given to right-adjust ``text``.  If
-            unspecified, the width of the attached terminal is used (default).
+        :arg int width: Total width given to right-adjust ``text``.
         :arg str fillchar: String for padding left-of ``text``.
         :returns: String of ``text``, right-aligned by ``width``.
         :rtype: str
         """
-        leftside = fillchar * int(
-            (max(0.0, float(width.__index__() - self.length()))) / float(len(fillchar)))
-        return ''.join((leftside, self))
+        return wcwidth_rjust(self, width.__index__(), fillchar, control_codes='ignore')
 
     def center(self, width: SupportsIndex, fillchar: str = ' ') -> str:
         """
         Return string containing sequences, centered.
 
-        :arg int width: Total width given to center ``text``.  If
-            unspecified, the width of the attached terminal is used (default).
+        :arg int width: Total width given to center ``text``.
         :arg str fillchar: String for padding left and right-of ``text``.
         :returns: String of ``text``, centered by ``width``.
         :rtype: str
         """
-        split = max(0.0, float(width.__index__()) - self.length()) / 2
-        leftside = fillchar * int(
-            (max(0.0, math.floor(split))) / float(len(fillchar)))
-        rightside = fillchar * int(
-            (max(0.0, math.ceil(split))) / float(len(fillchar)))
-        return ''.join((leftside, self, rightside))
+        return wcwidth_center(self, width.__index__(), fillchar, control_codes='ignore')
 
     def truncate(self, width: SupportsIndex) -> str:
         """
         Truncate a string in a sequence-aware manner.
 
         Any printable characters beyond ``width`` are removed, while all
-        sequences remain in place. Horizontal Sequences are first expanded
+        sequences remain in place. Horizontal sequences are first expanded
         by :meth:`padd`.
 
+        Wide characters (such as CJK or emoji) that would partially exceed
+        ``width`` are replaced with space padding to maintain exact width.
+
+        SGR (terminal styling) sequences are propagated: the result begins
+        with any active style at the start position and ends with a reset
+        sequence if styles were active.
+
         :arg int width: The printable width to truncate the string to.
         :rtype: str
-        :returns: String truncated to at most ``width`` printable characters.
+        :returns: String truncated to exactly ``width`` printable characters.
         """
-        output = ""
-        current_width = 0
-        target_width = width.__index__()
-        parsed_seq = iter_parse(self._term, self.padd())
-
-        # Retain all text until non-cap width reaches desired width
-        for text, cap in parsed_seq:
-            if not cap:
-                # use wcwidth clipped to 0 because it can sometimes return -1
-                current_width += max(wcwidth(text), 0)
-                if current_width > target_width:
-                    break
-            output += text
-
-        # Return with remaining caps appended
-        return f'{output}{"".join(text for text, cap in parsed_seq if cap)}'
+        # Use padd() to expand terminal-specific cursor movements to spaces,
+        # then use wcwidth's clip() to truncate while preserving all sequences.
+        return wcwidth_clip(self.padd(), 0, width.__index__())
 
     def length(self) -> int:
         r"""
         Return the printable length of string containing sequences.
 
-        Strings containing ``term.left`` or ``\b`` will cause "overstrike",
-        but a length less than 0 is not ever returned. So ``_\b+`` is a
-        length of 1 (displays as ``+``), but ``\b`` alone is simply a
-        length of 0.
+        Returns the maximum horizontal cursor extent reached while processing
+        the string. Backspace and cursor-left movements do not reduce the
+        length below the maximum position reached.
 
-        Some characters may consume more than one cell, mainly those CJK
-        Unified Ideographs (Chinese, Japanese, Korean) defined by Unicode
-        as half or full-width characters.
+        Some characters may consume more than one cell, mainly CJK (Chinese,
+        Japanese, Korean) and Emojis and some kinds of symbols.
 
         For example:
 
@@ -379,8 +248,7 @@ class Sequence(str):
             as ``term.clear`` will not give accurate returns, it is not
             considered lengthy (a length of 0).
         """
-        # because control characters may return -1, "clip" their length to 0.
-        return sum(max(wcwidth(w_char), 0) for w_char in self.padd(strip=True))
+        return wcwidth_width(self)
 
     def strip(self, chars: Optional[str] = None) -> str:
         """
@@ -439,11 +307,16 @@ class Sequence(str):
             if self._term.caps_compiled.search(data) is None:
                 return str(data)
 
+            # pylint: disable-next=protected-access
+            caps = self._term._hdist_caps_named_compiled
+        else:
+            # pylint: disable-next=protected-access
+            caps = self._term._caps_named_compiled
+
         outp = ''
         last_end = 0
 
-        # pylint: disable-next=protected-access
-        for match in self._term._caps_named_compiled.finditer(data):
+        for match in caps.finditer(data):
 
             # Capture unmatched text between matched capabilities
             if match.start() > last_end:
@@ -474,6 +347,10 @@ def iter_parse(term: 'Terminal', text: str) -> Iterator[Tuple[str, Optional[Term
     value for ``capability`` may be ``None``, where ``text`` is
     :class:`str` of length 1.  Otherwise, ``text`` is a full
     matching sequence of given capability.
+
+    .. note:: Previously used by SequenceTextWrapper, sequence-aware text
+       wrapping now exists in wcwidth as :func:`wcwidth.wrap`. This function
+       is kept for API compatibility and used by :func:`measure_length`.
     """
     for match in term._caps_compiled_any.finditer(text):  # pylint: disable=protected-access
         name = match.lastgroup
@@ -486,10 +363,12 @@ def iter_parse(term: 'Terminal', text: str) -> Iterator[Tuple[str, Optional[Term
 
 def measure_length(text: str, term: 'Terminal') -> int:
     """
-    .. deprecated:: 1.12.0.
+    Kept for API compatibility.
 
     :rtype: int
     :returns: Length of the first sequence in the string
+
+    .. deprecated:: 1.30.0
     """
     try:
         text, capability = next(iter_parse(term, text))
