@@ -16,7 +16,8 @@ from .accessories import (
     TestTerminal,
     as_subprocess,
     read_until_semaphore,
-    pty_test
+    pty_test,
+    PCT_MAXWAIT_KEYSTROKE,
 )
 
 pytestmark = pytest.mark.skipif(
@@ -85,15 +86,64 @@ def test_does_sixel_not_a_tty():
     child()
 
 
+def test_get_cell_height_and_width_success():
+    """get_cell_height_and_width returns expected tuple with valid response."""
+    def child(term):
+        term.ungetch('\x1b[6;16;8t')
+        result = term.get_cell_height_and_width(timeout=0.01)
+        assert isinstance(result, tuple)
+        assert len(result) == 2
+        assert result == (16, 8)
+        return b'OK'
+
+    output = pty_test(child, parent_func=None,
+                      test_name='test_get_cell_height_and_width_success')
+    assert 'OK' in output
+
+
+def test_get_cell_height_and_width_timeout():
+    """get_cell_height_and_width returns (-1, -1) on timeout."""
+    def child(term):
+        result = term.get_cell_height_and_width(timeout=0.01)
+        assert isinstance(result, tuple)
+        assert len(result) == 2
+        assert result == (-1, -1)
+        return b'OK'
+
+    output = pty_test(child, parent_func=None,
+                      test_name='test_get_cell_height_and_width_timeout')
+    assert 'OK' in output
+
+
+def test_get_cell_height_and_width_caching():
+    """get_cell_height_and_width caches results unless force=True."""
+    def child(term):
+        term.ungetch('\x1b[6;20;10t')
+        result1 = term.get_cell_height_and_width(timeout=0.01)
+        assert result1 == (20, 10)
+
+        result2 = term.get_cell_height_and_width(timeout=0.01)
+        assert result2 == (20, 10)
+
+        result3 = term.get_cell_height_and_width(timeout=0.01, force=True)
+        assert result3 == (-1, -1)
+        return b'OK'
+
+    output = pty_test(child, parent_func=None,
+                      test_name='test_get_cell_height_and_width_caching')
+    assert 'OK' in output
+
+
 def test_get_sixel_height_and_width_0s_ungetch():
     """0-second get_sixel_height_and_width call with mocked response via ungetch."""
     def child(term):
         stime = time.time()
-        term.ungetch('\x1b[?2;0;800;600S')
+        # XTWINOPS 16t first: cell is 16x8, with 24x80 terminal = 384x640
+        term.ungetch('\x1b[6;16;8t')
 
         height, width = term.get_sixel_height_and_width(timeout=0.01)
         assert math.floor(time.time() - stime) == 0.0
-        assert (height, width) == (600, 800)
+        assert (height, width) == (384, 640)
         return b'OK'
 
     output = pty_test(child, parent_func=None,
@@ -105,7 +155,6 @@ def test_get_sixel_height_and_width_0s_ungetch():
 @pytest.mark.parametrize('method_name,expected_result,max_time', [
     ('get_sixel_height_and_width', (-1, -1), 0.15),
     ('get_sixel_colors', -1, 0.18),  # Longer: queries XTSMGRAPHICS + DA1
-    ('get_cell_height_and_width', (-1, -1), 0.15),
 ])
 def test_sixel_methods_timeout(method_name, expected_result, max_time):
     """Sixel query methods return failure values on timeout."""
@@ -137,25 +186,21 @@ def test_get_sixel_height_and_width_invalid_response():
     assert 'OK' in output
 
 
-@pytest.mark.parametrize('method_name,ungetch_response,expected_result', [
-    ('get_sixel_colors', '\x1b[?1;0;256S', 256),
-    ('get_cell_height_and_width', '\x1b[6;16;8t', (16, 8)),
-])
-def test_sixel_query_methods_success(method_name, ungetch_response, expected_result):
-    """Sixel query methods return expected values with valid responses."""
+def test_get_sixel_colors_success():
+    """get_sixel_colors returns expected value with valid response."""
     def child(term):
-        term.ungetch(ungetch_response)
-        result = getattr(term, method_name)(timeout=0.01)
-        assert result == expected_result
+        term.ungetch('\x1b[?1;0;256S')
+        result = term.get_sixel_colors(timeout=0.01)
+        assert result == 256
         return b'OK'
 
-    output = pty_test(child, parent_func=None, test_name=f'test_{method_name}_success')
+    output = pty_test(child, parent_func=None, test_name='test_get_sixel_colors_success')
     assert 'OK' in output
 
 
 @pytest.mark.skipif(TEST_QUICK, reason="TEST_QUICK specified")
-def test_sixel_height_and_width_xtsmgraphics_success():
-    """Test sixel height and width succeeds quickly with XTSMGRAPHICS response."""
+def test_sixel_height_and_width_xtwinops_cell_success():
+    """Test sixel height and width succeeds quickly with XTWINOPS 16t response."""
     def child(term):
         os.write(sys.__stdout__.fileno(), SEMAPHORE)
         with term.cbreak():
@@ -167,16 +212,18 @@ def test_sixel_height_and_width_xtsmgraphics_success():
 
     def parent(master_fd):
         read_until_semaphore(master_fd)
-        # Read and discard the query sequence
+        # Read and discard the query sequence (XTWINOPS 16t)
         os.read(master_fd, 100)
-        # Respond immediately with XTSMGRAPHICS
-        os.write(master_fd, b'\x1b[?2;0;1024;768S')
+        # Respond immediately with XTWINOPS 16t: cell is 16x8 pixels
+        # Terminal is typically 24 rows x 80 cols, so 384x640 total pixels
+        os.write(master_fd, b'\x1b[6;16;8t')
 
     stime = time.time()
-    output = pty_test(child, parent, 'test_sixel_height_and_width_xtsmgraphics_success')
+    output = pty_test(child, parent, 'test_sixel_height_and_width_xtwinops_cell_success')
     dimensions, duration = output.split('|')
 
-    assert dimensions == '768x1024'
+    # Assuming 24 rows x 80 cols: 16*24 = 384 height, 8*80 = 640 width
+    assert dimensions == '384x640'
     # Should complete very quickly (not wait for fallback)
     assert float(duration) < 0.2
     assert math.floor(time.time() - stime) == 0.0
@@ -184,7 +231,7 @@ def test_sixel_height_and_width_xtsmgraphics_success():
 
 @pytest.mark.skipif(TEST_QUICK, reason="TEST_QUICK specified")
 def test_sixel_height_and_width_fallback_to_xtwinops():
-    """Test sixel height and width falls back to XTWINOPS after XTSMGRAPHICS timeout."""
+    """Test sixel height and width falls back to XTWINOPS 14t after 16t timeout."""
     def child(term):
         os.write(sys.__stdout__.fileno(), SEMAPHORE)
         with term.cbreak():
@@ -196,11 +243,11 @@ def test_sixel_height_and_width_fallback_to_xtwinops():
 
     def parent(master_fd):
         read_until_semaphore(master_fd)
-        # Read and discard first query (XTSMGRAPHICS)
+        # Read and discard first query (XTWINOPS 16t - cell size)
         os.read(master_fd, 100)
-        # Wait for XTSMGRAPHICS timeout, then read XTWINOPS query
-        time.sleep(0.55)  # timeout/2 = 0.5, add a bit
-        os.read(master_fd, 100)  # Read XTWINOPS query
+        # Wait for XTWINOPS 16t timeout, then read XTWINOPS 14t query
+        time.sleep(0.36)  # timeout/3, add a bit
+        os.read(master_fd, 100)  # Read XTWINOPS 14t query
         # Respond to XTWINOPS 14t (window size)
         os.write(master_fd, b'\x1b[4;600;800t')
 
@@ -209,47 +256,53 @@ def test_sixel_height_and_width_fallback_to_xtwinops():
     dimensions, duration = output.split('|')
 
     assert dimensions == '600x800'
-    # Should take around timeout/2 (0.5s) + response time
-    assert 0.5 <= float(duration) <= 0.7
+    # Should take around timeout/3 + a bit
+    assert 0.30 <= float(duration) <= 0.5
     assert math.floor(time.time() - stime) == 0.0
 
 
 @pytest.mark.skipif(TEST_QUICK, reason="TEST_QUICK specified")
 def test_sixel_height_and_width_both_timeout():
-    """Test sixel height and width returns (-1, -1) when both methods timeout."""
+    """Test sixel height and width returns (-1, -1) when all methods timeout."""
+
+    timeout = 1.0
+
     def child(term):
         os.write(sys.__stdout__.fileno(), SEMAPHORE)
         with term.cbreak():
             stime = time.time()
-            height, width = term.get_sixel_height_and_width(timeout=1.0, force=True)
+            height, width = term.get_sixel_height_and_width(timeout=timeout, force=True)
             duration_s = time.time() - stime
             result = f'{height}x{width}|{duration_s:.2f}'
             return result.encode('utf-8')
 
     def parent(master_fd):
         read_until_semaphore(master_fd)
-        # Read and discard first query (XTSMGRAPHICS)
+        # Read and discard first query (XTWINOPS 16t)
         os.read(master_fd, 100)
-        # Wait for XTSMGRAPHICS timeout, then read XTWINOPS query
-        time.sleep(0.55)  # timeout/2 = 0.5, add a bit
-        os.read(master_fd, 100)  # Read XTWINOPS query
-        # Don't respond - let XTWINOPS timeout too
+        # Wait for first timeout, then read XTWINOPS 14t query
+        time.sleep(timeout * .36)  # timeout/3, add a bit
+        os.read(master_fd, 100)  # Read XTWINOPS 14t query
+        # Wait for second timeout, read XTSMGRAPHICS query
+        time.sleep(timeout * .36)
+        os.read(master_fd, 100)  # Read XTSMGRAPHICS query
+        # Don't respond - cause condition that all queries timeout
 
     stime = time.time()
     output = pty_test(child, parent, 'test_sixel_height_and_width_both_timeout')
     dimensions, duration = output.split('|')
 
     assert dimensions == '-1x-1'
-    # Should take close to full timeout (1.0s)
-    assert 0.9 <= float(duration) <= 1.2
-    assert math.floor(time.time() - stime) == 1.0
+    # Should take around timeout/3 * 3 = 1.0s for the three queries
+    assert 0.90 <= float(duration) <= timeout * PCT_MAXWAIT_KEYSTROKE
+    assert math.floor(time.time() - stime) <= timeout * PCT_MAXWAIT_KEYSTROKE
 
 
-@pytest.mark.parametrize('method_name,ungetch_response,expected_result', [
-    ('get_sixel_height_and_width', '\x1b[?2;0;640;480S', (480, 640)),
-    ('get_cell_height_and_width', '\x1b[6;20;10t', (20, 10)),
+@pytest.mark.parametrize('method_name,ungetch_response,expected_result,expected_failure', [
+    ('get_sixel_height_and_width', '\x1b[6;20;8t', (480, 640), (-1, -1)),
+    ('get_sixel_colors', '\x1b[?1;0;256S', 256, -1),
 ])
-def test_sixel_methods_caching(method_name, ungetch_response, expected_result):
+def test_sixel_methods_caching(method_name, ungetch_response, expected_result, expected_failure):
     """Sixel query methods cache results unless force=True."""
     def child(term):
         term.ungetch(ungetch_response)
@@ -260,7 +313,7 @@ def test_sixel_methods_caching(method_name, ungetch_response, expected_result):
         assert result2 == expected_result
 
         result3 = getattr(term, method_name)(timeout=0.01, force=True)
-        assert result3 == (-1, -1)
+        assert result3 == expected_failure
         return b'OK'
 
     output = pty_test(child, parent_func=None,
@@ -268,34 +321,17 @@ def test_sixel_methods_caching(method_name, ungetch_response, expected_result):
     assert 'OK' in output
 
 
-def test_get_sixel_colors_caching():
-    """Test that sixel colors are cached unless force=True."""
-    def child(term):
-        term.ungetch('\x1b[?1;0;256S')
-        colors1 = term.get_sixel_colors(timeout=0.01)
-        assert colors1 == 256
-
-        colors2 = term.get_sixel_colors(timeout=0.01)
-        assert colors2 == 256
-
-        colors3 = term.get_sixel_colors(timeout=0.01, force=True)
-        assert colors3 == -1
-        return b'OK'
-
-    output = pty_test(child, parent_func=None, test_name='test_get_sixel_colors_caching')
-    assert 'OK' in output
-
-
 def test_timeout_reduction_subprocess():
-    """Test timeout reduction path when XTSMGRAPHICS fails (subprocess version)."""
+    """Test timeout path when all methods fail (subprocess version)."""
     def child(term):
-        # Call with a real timeout to trigger the timeout reduction path
-        # Both XTSMGRAPHICS and XTWINOPS will timeout (no ungetch)
+        # Call with a real timeout to trigger the timeout path
+        # All methods will timeout (no ungetch)
         result = term.get_sixel_height_and_width(timeout=0.2, force=True)
         assert result == (-1, -1)
-        # Should cache failure in both caches
-        assert term._xtsmgraphics_cache == (-1, -1)
+        # Should cache failure in all caches
+        assert term._xtwinops_cell_cache == (-1, -1)
         assert term._xtwinops_cache == (-1, -1)
+        assert term._xtsmgraphics_cache == (-1, -1)
         return b'OK'
 
     output = pty_test(child, parent_func=None, test_name='test_timeout_reduction_subprocess')
@@ -303,48 +339,53 @@ def test_timeout_reduction_subprocess():
 
 
 @pytest.mark.skipif(TEST_QUICK, reason="TEST_QUICK specified")
-def test_timeout_reduction_after_xtsmgraphics_fails():
-    """Test timeout is reduced after XTSMGRAPHICS times out and fallback uses reduced timeout."""
+def test_timeout_allocation_across_methods():
+    """Test timeout is allocated across detection methods."""
     def child(term):
         os.write(sys.__stdout__.fileno(), SEMAPHORE)
         with term.cbreak():
             stime = time.time()
-            result = term.get_sixel_height_and_width(timeout=0.6, force=True)
+            result = term.get_sixel_height_and_width(timeout=0.8, force=True)
             elapsed = time.time() - stime
 
-            cached_value = term._xtsmgraphics_cache
+            cell_cached = term._xtwinops_cell_cache
+            window_cached = term._xtwinops_cache
             result_str = (f'{result[0]}x{result[1]}|{elapsed:.2f}|'
-                          f'{cached_value[0]}x{cached_value[1]}')
+                          f'{cell_cached[0]}x{cell_cached[1]}|'
+                          f'{window_cached[0]}x{window_cached[1]}')
             return result_str.encode('utf-8')
 
     def parent(master_fd):
         read_until_semaphore(master_fd)
+        os.read(master_fd, 100)  # Read XTWINOPS 16t query
+        time.sleep(0.29)  # Wait for first timeout (timeout/3 ≈ 0.267 seconds)
+        os.read(master_fd, 100)  # Read XTWINOPS 14t query
+        time.sleep(0.29)  # Wait for second timeout
         os.read(master_fd, 100)  # Read XTSMGRAPHICS query
-        time.sleep(0.33)  # Wait for first timeout (timeout/2 = 0.3 seconds)
-        os.read(master_fd, 100)  # Read XTWINOPS query
 
-    output = pty_test(child, parent, 'test_timeout_reduction_after_xtsmgraphics_fails')
-    result, elapsed, cached = output.split('|')
+    output = pty_test(child, parent, 'test_timeout_allocation_across_methods')
+    result, elapsed, cell_cached, window_cached = output.split('|')
 
     assert result == '-1x-1'
-    # Should take close to full timeout (0.6s) - XTSMGRAPHICS gets 0.3s, XTWINOPS gets ~0.3s
-    assert 0.55 <= float(elapsed) <= 0.7
-    # Should cache the failure
-    assert cached == '-1x-1'
+    # Should take around 3 * timeout/3 = 0.8s (3 queries at third timeout each)
+    assert 0.75 <= float(elapsed) <= 0.95
+    # Should cache the failures
+    assert cell_cached == '-1x-1'
+    assert window_cached == '-1x-1'
 
 
 @pytest.mark.skipif(TEST_QUICK, reason="TEST_QUICK specified")
-def test_xtsmgraphics_sticky_failure():
-    """Test that XTSMGRAPHICS failure sets sticky flag and skips to XTWINOPS."""
+def test_cell_cache_sticky_failure():
+    """Test that cell cache failure is cached but can be bypassed with force=True."""
     def child(term):
         os.write(sys.__stdout__.fileno(), SEMAPHORE)
         with term.cbreak():
-            # First call - XTSMGRAPHICS will timeout, fallback to XTWINOPS
+            # First call - cell will timeout, fallback to window
             stime1 = time.time()
             height1, width1 = term.get_sixel_height_and_width(timeout=1.0, force=True)
             duration1 = time.time() - stime1
 
-            # Second call - should skip XTSMGRAPHICS (sticky failure) and go directly to XTWINOPS
+            # Second call - should skip cell (sticky failure) and go directly to window
             stime2 = time.time()
             height2, width2 = term.get_sixel_height_and_width(timeout=1.0, force=True)
             duration2 = time.time() - stime2
@@ -355,34 +396,34 @@ def test_xtsmgraphics_sticky_failure():
     def parent(master_fd):
         read_until_semaphore(master_fd)
 
-        # First query - read XTSMGRAPHICS, wait for timeout, read XTWINOPS, respond
-        os.read(master_fd, 100)  # XTSMGRAPHICS query
-        time.sleep(0.55)  # timeout/2 = 0.5, add a bit
-        os.read(master_fd, 100)  # XTWINOPS query after fallback
+        # First query - read cell, wait for timeout, read window, respond
+        os.read(master_fd, 100)  # XTWINOPS 16t query
+        time.sleep(0.36)  # timeout/3 ≈ 0.33s, add a bit
+        os.read(master_fd, 100)  # XTWINOPS 14t query after fallback
         os.write(master_fd, b'\x1b[4;480;640t')
 
-        # Second query - should only be XTWINOPS (sticky failure skips XTSMGRAPHICS)
-        os.read(master_fd, 100)  # XTWINOPS query
-        time.sleep(0.05)
+        # Second query with force=True - re-queries cell, then window
+        os.read(master_fd, 100)  # XTWINOPS 16t query (force=True bypasses sticky failure)
+        time.sleep(0.36)  # timeout/3 ≈ 0.33s, add a bit
+        os.read(master_fd, 100)  # XTWINOPS 14t query after fallback
         os.write(master_fd, b'\x1b[4;480;640t')
 
-    output = pty_test(child, parent, 'test_xtsmgraphics_sticky_failure')
+    output = pty_test(child, parent, 'test_cell_cache_sticky_failure')
     dim1, dur1, dim2, dur2 = output.split('|')
 
-    # First call should fallback (takes ~timeout/2)
+    # First call should fallback (takes ~timeout/3)
     assert dim1 == '480x640'
-    assert 0.5 <= float(dur1) <= 0.7
+    assert 0.30 <= float(dur1) <= 0.5
 
-    # Second call should skip XTSMGRAPHICS and go directly to XTWINOPS (fast)
+    # Second call with force=True also re-queries and fallbacks (takes ~timeout/3)
     assert dim2 == '480x640'
-    assert float(dur2) < 0.2  # Much faster than first call
+    assert 0.30 <= float(dur2) <= 0.5
 
 
 @pytest.mark.skipif(TEST_QUICK, reason="TEST_QUICK specified")
 @pytest.mark.parametrize('method_name,expected_failure,max_time', [
     ('get_sixel_height_and_width', (-1, -1), 0.15),
     ('get_sixel_colors', -1, 0.18),  # Longer: queries XTSMGRAPHICS + DA1
-    ('get_cell_height_and_width', (-1, -1), 0.15),
 ])
 def test_cached_failure_returns_immediately(method_name, expected_failure, max_time):
     """Test that cached failure results return immediately on subsequent calls."""
@@ -401,12 +442,12 @@ def test_cached_failure_returns_immediately(method_name, expected_failure, max_t
         assert result2 == expected_failure
         assert elapsed2 < 0.01  # Should be instant from cache
 
-        # Third call with force=True - should bypass cache and timeout again
+        # Third call with force=True - bypasses cache and re-queries
         stime3 = time.time()
         result3 = getattr(term, method_name)(timeout=0.1, force=True)
         elapsed3 = time.time() - stime3
         assert result3 == expected_failure
-        assert 0.08 <= elapsed3 <= max_time
+        assert 0.08 <= elapsed3 <= max_time  # Should timeout again
         return b'OK'
 
     output = pty_test(child, parent_func=None,
@@ -434,41 +475,55 @@ def test_get_sixel_colors_fallback_to_da1(da1_response, expected_colors):
     assert 'OK' in output
 
 
-def test_konsole_bogus_xtsmgraphics_validates_with_xtwinops():
-    """Test that bogus XTSMGRAPHICS values (like Konsole's 16384x16384) validate with XTWINOPS."""
+def test_sixel_height_width_with_xtwinops_14t():
+    """Test that XTWINOPS 14t fallback works when cell size query fails."""
     def child(term):
-        # Simulate Konsole: XTSMGRAPHICS returns 16384x16384 (bogus),
-        # then XTWINOPS returns real size
-        # ungetch in reverse order - XTWINOPS response first, then XTSMGRAPHICS
+        # XTWINOPS 16t fails (no response), XTWINOPS 14t succeeds
         term.ungetch('\x1b[4;1080;1920t')  # XTWINOPS 14t response: 1920x1080 window
-        term.ungetch('\x1b[?2;0;16384;16384S')  # XTSMGRAPHICS response: bogus 16384x16384
 
         height, width = term.get_sixel_height_and_width(timeout=0.1)
 
-        # Should use XTWINOPS value, not the bogus XTSMGRAPHICS value
+        # Should use XTWINOPS 14t value
         assert (height, width) == (1080, 1920)
-        # XTWINOPS result should be cached
+        # XTWINOPS 14t result should be cached
         assert term._xtwinops_cache == (1080, 1920)
-        # XTSMGRAPHICS detected Konsole's bogus 16384x16384 and cached failure
-        assert term._xtsmgraphics_cache == (-1, -1)
+        # XTWINOPS 16t should have cached failure
+        assert term._xtwinops_cell_cache == (-1, -1)
         return b'OK'
 
     output = pty_test(child, parent_func=None,
-                      test_name='test_konsole_bogus_xtsmgraphics_validates_with_xtwinops')
+                      test_name='test_sixel_height_width_with_xtwinops_14t')
     assert 'OK' in output
 
 
+def test_konsole_bogus_xtsmgraphics_16384():
+    """Test that bogus XTSMGRAPHICS values (like Konsole's 16384x16384) are rejected."""
+    def child(term):
+        # Note that real Konsole returns the correct values, answering to
+        # XTWINOPS, this is just a "theoretical" terminal -- for code coverage.
+        term.ungetch('\x1b[?2;0;16384;16384S')  # XTSMGRAPHICS response: bogus 16384x16384
+        # Pre-cache failures to skip directly to XTSMGRAPHICS
+        term._xtwinops_cell_cache = (-1, -1)
+        term._xtwinops_cache = (-1, -1)
+
+        height, width = term.get_sixel_height_and_width(timeout=0.1)
+
+        # Should reject the bogus 16384x16384 value and return failure
+        # XTSMGRAPHICS detected Konsole's bogus 16384x16384 and cached failure
+        assert (height, width) == (-1, -1)
+        assert term._xtsmgraphics_cache == (-1, -1)
+
+    pty_test(child, parent_func=None, test_name='test_konsole_bogus_xtsmgraphics_16384')
+
+
 @pytest.mark.skipif(TEST_QUICK, reason="TEST_QUICK specified")
 def test_konsole_bogus_xtsmgraphics_real_terminal():
-    """Test Konsole-like behavior with real PTY to verify both queries are made."""
+    """Test _get_xtsmgraphics rejects Konsole's bogus 16384x16384 value."""
     def child(term):
         os.write(sys.__stdout__.fileno(), SEMAPHORE)
         with term.cbreak():
-            stime = time.time()
-            height, width = term.get_sixel_height_and_width(timeout=1.0, force=True)
-            duration_s = time.time() - stime
-            result = f'{height}x{width}|{duration_s:.2f}'
-            return result.encode('utf-8')
+            result = term._get_xtsmgraphics(timeout=0.5)
+            return f'{result[0]}x{result[1]}'.encode('utf-8')
 
     def parent(master_fd):
         read_until_semaphore(master_fd)
@@ -476,32 +531,23 @@ def test_konsole_bogus_xtsmgraphics_real_terminal():
         os.read(master_fd, 100)
         # Respond with bogus Konsole-like value
         os.write(master_fd, b'\x1b[?2;0;16384;16384S')
-        # Give child time to process, then read XTWINOPS validation query
-        time.sleep(0.02)
-        os.read(master_fd, 100)
-        # Respond with realistic window size
-        os.write(master_fd, b'\x1b[4;1080;1920t')
 
     output = pty_test(child, parent, 'test_konsole_bogus_xtsmgraphics_real_terminal')
-    dimensions, duration = output.split('|')
 
-    # Should use XTWINOPS value, not bogus XTSMGRAPHICS
-    assert dimensions == '1080x1920'
-    # Should complete quickly (both queries succeed)
-    assert float(duration) < 0.3
+    # Should reject the bogus 16384x16384 value and return failure
+    assert output == '-1x-1'
 
 
-@pytest.mark.parametrize('xtsmgraphics_cache,xtwinops_cache,expected_result', [
-    ((768, 1024), (1080, 1920), (768, 1024)),  # XTSMGRAPHICS succeeded
-    ((-1, -1), (1080, 1920), (1080, 1920)),    # XTSMGRAPHICS failed, XTWINOPS succeeded
+@pytest.mark.parametrize('cell_cache,window_cache,expected_result', [
+    ((16, 8), None, (384, 640)),      # Cell size: 16*24=384, 8*80=640
+    ((-1, -1), (1080, 1920), (1080, 1920)),  # Cell failed, window succeeded
 ])
-def test_fast_path_with_both_caches_populated(xtsmgraphics_cache,
-                                              xtwinops_cache,
-                                              expected_result):
-    """Test fast path returns cached value instantly when both caches populated."""
+def test_fast_path_with_caches_populated(cell_cache, window_cache, expected_result):
+    """Test returns cached value instantly when caches populated."""
     def child(term):
-        term._xtsmgraphics_cache = xtsmgraphics_cache
-        term._xtwinops_cache = xtwinops_cache
+        term._xtwinops_cell_cache = cell_cache
+        term._xtwinops_cache = window_cache
+        term._xtsmgraphics_cache = None
 
         stime = time.time()
         height, width = term.get_sixel_height_and_width(timeout=0.1)
@@ -516,17 +562,17 @@ def test_fast_path_with_both_caches_populated(xtsmgraphics_cache,
     assert 'OK' in output
 
 
-def test_xtwinops_cache_return_after_xtsmgraphics_fails():
-    """Test returning cached XTWINOPS value when XTSMGRAPHICS fails."""
+def test_xtwinops_window_cache_after_cell_fails():
+    """Test returning cached XTWINOPS 14t value when 16t fails."""
     def child(term):
-        # First call: XTSMGRAPHICS will fail, XTWINOPS will succeed
-        term.ungetch('\x1b[4;1080;1920t')  # XTWINOPS response
+        # First call: XTWINOPS 16t will fail, XTWINOPS 14t will succeed
+        term.ungetch('\x1b[4;1080;1920t')  # XTWINOPS 14t response
         result1 = term.get_sixel_height_and_width(timeout=0.1)
         assert result1 == (1080, 1920)
-        assert term._xtsmgraphics_cache == (-1, -1)
+        assert term._xtwinops_cell_cache == (-1, -1)
         assert term._xtwinops_cache == (1080, 1920)
 
-        # Second call: Should return cached XTWINOPS without re-querying
+        # Second call: Should return cached XTWINOPS 14t without re-querying
         stime = time.time()
         result2 = term.get_sixel_height_and_width(timeout=0.1)
         elapsed = time.time() - stime
@@ -536,65 +582,200 @@ def test_xtwinops_cache_return_after_xtsmgraphics_fails():
         return b'OK'
 
     output = pty_test(child, parent_func=None,
-                      test_name='test_xtwinops_cache_return_after_xtsmgraphics_fails')
+                      test_name='test_xtwinops_window_cache_after_cell_fails')
     assert 'OK' in output
 
 
-def test_xtsmgraphics_cache_hit_without_xtwinops_cache():
-    """XTSMGRAPHICS cache hit when XTWINOPS cache is None."""
+def test_force_query_with_existing_cache():
+    """Force query even with existing cache."""
     def child(term):
-        # Prepopulate only XTSMGRAPHICS cache with successful value
-        term._xtsmgraphics_cache = (768, 1024)
-        term._xtwinops_cache = None
-
-        stime = time.time()
-        height, width = term.get_sixel_height_and_width(timeout=0.1)
-        elapsed = time.time() - stime
-
-        assert (height, width) == (768, 1024)
-        assert elapsed < 0.01
-        return b'OK'
-
-    output = pty_test(child, parent_func=None,
-                      test_name='test_xtsmgraphics_cache_hit_without_xtwinops_cache')
-    assert 'OK' in output
-
-
-def test_force_xtwinops_query_with_existing_cache():
-    """Force XTWINOPS query even with existing cache."""
-    def child(term):
-        # Prepopulate both caches with XTSMGRAPHICS failed
-        term._xtsmgraphics_cache = (-1, -1)
+        # Prepopulate all caches
+        term._xtwinops_cell_cache = (16, 8)
         term._xtwinops_cache = (1080, 1920)
+        term._xtsmgraphics_cache = (-1, -1)
 
-        # Call with force=True - should re-query XTWINOPS, no ungetch so query will timeout
+        # Call with force=True - should re-query, no ungetch so query will timeout
         result = term.get_sixel_height_and_width(timeout=0.1, force=True)
 
         # Should get timeout result, not cached value
         assert result == (-1, -1)
-        assert term._xtwinops_cache == (-1, -1)
+        assert term._xtwinops_cell_cache == (-1, -1)
         return b'OK'
 
     output = pty_test(child, parent_func=None,
-                      test_name='test_force_xtwinops_query_with_existing_cache')
+                      test_name='test_force_query_with_existing_cache')
     assert 'OK' in output
 
 
-def test_xtwinops_cache_return_on_xtsmgraphics_query_fails():
-    """Return cached XTWINOPS when XTSMGRAPHICS query fails."""
+def test_window_cache_return_when_cell_query_fails():
+    """Return cached XTWINOPS 14t when cell query fails, caching the failure."""
     def child(term):
-        # Prepopulate only XTWINOPS cache, XTSMGRAPHICS cache is None
-        term._xtsmgraphics_cache = None
+        # Prepopulate only window cache, cell cache is None
+        term._xtwinops_cell_cache = None
         term._xtwinops_cache = (1080, 1920)
+        term._xtsmgraphics_cache = None
 
-        # Call without force - XTSMGRAPHICS will be queried, fail (no ungetch),
-        # then return cached XTWINOPS
+        # Call without force - cell query fails and caches failure, then returns window cache
         height, width = term.get_sixel_height_and_width(timeout=0.1)
 
         assert (height, width) == (1080, 1920)
-        assert term._xtsmgraphics_cache == (-1, -1)
+        # Cell cache now has cached failure from the query attempt
+        assert term._xtwinops_cell_cache == (-1, -1)
         return b'OK'
 
     output = pty_test(child, parent_func=None,
-                      test_name='test_xtwinops_cache_return_on_xtsmgraphics_query_fails')
+                      test_name='test_window_cache_return_when_cell_query_fails')
     assert 'OK' in output
+
+
+def test_preferred_size_cache_path():
+    """Test get_sixel_height_and_width falls back to preferred_size_cache as last resort."""
+    @as_subprocess
+    def child():
+        from blessed.terminal import WINSZ
+        term = TestTerminal()
+
+        # Pre-cache failures for methods 1-3 so it falls through to method 4
+        term._xtwinops_cell_cache = (-1, -1)
+        term._xtwinops_cache = (-1, -1)
+        term._xtsmgraphics_cache = (-1, -1)
+        term._preferred_size_cache = WINSZ(ws_row=24, ws_col=80, ws_xpixel=1920, ws_ypixel=1080)
+
+        result = term.get_sixel_height_and_width(timeout=0.1)
+        assert result == (1080, 1920)
+    child()
+
+
+def test_preferred_size_cache_with_zero_pixels():
+    """Test that zero pixel dimensions in preferred_size_cache are skipped."""
+    def child(term):
+        from blessed.terminal import WINSZ
+
+        term._preferred_size_cache = WINSZ(ws_row=24, ws_col=80, ws_xpixel=0, ws_ypixel=0)
+        term.ungetch('\x1b[6;16;8t')  # Cell size response
+
+        result = term.get_sixel_height_and_width(timeout=0.1)
+        assert result == (384, 640)  # Falls through to cell query
+        return b'OK'
+
+    output = pty_test(child, parent_func=None,
+                      test_name='test_preferred_size_cache_with_zero_pixels')
+    assert 'OK' in output
+
+
+@pytest.mark.parametrize('ws_ypixel,ws_xpixel', [
+    (1920, 0),
+    (0, 1080),
+])
+def test_preferred_size_cache_with_partial_zero_pixels(ws_ypixel, ws_xpixel):
+    """Test that partial zero dimensions in preferred_size_cache are skipped."""
+    # this is something of a fictional terminal, though maybe for some it may
+    # someday be possible, very far down the logical fall-back ladder: that if
+    # any of either width or height is 0 through in-band resize protocol, that
+    # TIOCSWINSZ is used.
+    def child(term):
+        from blessed.terminal import WINSZ
+
+        # Pre-cache failures for methods 1-3 so it falls through to method 4
+        term._xtwinops_cell_cache = (-1, -1)
+        term._xtwinops_cache = (-1, -1)
+        term._xtsmgraphics_cache = (-1, -1)
+        term._preferred_size_cache = WINSZ(ws_row=24, ws_col=80,
+                                           ws_xpixel=ws_xpixel, ws_ypixel=ws_ypixel)
+
+        # Since preferred_size_cache has partial zero, should fall through to TIOCSWINSZ
+        # Mock TIOCSWINSZ to return valid dimensions
+        original_height_and_width = term._height_and_width
+
+        def mock_height_and_width():
+            orig = original_height_and_width()
+            return WINSZ(ws_row=orig.ws_row, ws_col=orig.ws_col,
+                         ws_xpixel=1600, ws_ypixel=900)
+
+        term._height_and_width = mock_height_and_width
+
+        result = term.get_sixel_height_and_width(timeout=0.1)
+        assert result == (900, 1600)  # Falls through to TIOCSWINSZ
+        return b'OK'
+
+    output = pty_test(child, parent_func=None,
+                      test_name=f'test_preferred_size_cache_partial_zero_{ws_ypixel}_{ws_xpixel}')
+    assert 'OK' in output
+
+
+def test_xtsmgraphics_cache_hit():
+    """Test XTSMGRAPHICS cache is returned when cell and window caches fail."""
+    def child(term):
+        # Pre-cache failures for cell and window, but success for XTSMGRAPHICS
+        term._xtwinops_cell_cache = (-1, -1)
+        term._xtwinops_cache = (-1, -1)
+        term._xtsmgraphics_cache = (600, 800)
+
+        # Should return cached XTSMGRAPHICS value instantly
+        stime = time.time()
+        result = term.get_sixel_height_and_width(timeout=0.1)
+        elapsed = time.time() - stime
+
+        assert result == (600, 800)
+        assert elapsed < 0.01  # Instant from cache
+        return b'OK'
+
+    output = pty_test(child, parent_func=None, test_name='test_xtsmgraphics_cache_hit')
+    assert 'OK' in output
+
+
+def test_tiocswinsz_path():
+    """Test TIOCSWINSZ path when all queries fail but ioctl returns pixel dimensions."""
+    def child(term):
+        from blessed.terminal import WINSZ
+        original_height_and_width = term._height_and_width
+
+        def mock_height_and_width():
+            orig = original_height_and_width()
+            return WINSZ(ws_row=orig.ws_row, ws_col=orig.ws_col,
+                         ws_xpixel=1600, ws_ypixel=900)
+
+        term._height_and_width = mock_height_and_width
+
+        result = term.get_sixel_height_and_width(timeout=0.1, force=True)
+        assert result == (900, 1600)
+        return b'OK'
+
+    output = pty_test(child, parent_func=None, test_name='test_tiocswinsz_path')
+    assert 'OK' in output
+
+
+def test_tiocswinsz_invalid_dimensions():
+    """Test TIOCSWINSZ path rejects invalid (zero or too large) pixel dimensions."""
+    def child(term):
+        from blessed.terminal import WINSZ
+        original_height_and_width = term._height_and_width
+
+        def mock_height_and_width():
+            orig = original_height_and_width()
+            return WINSZ(ws_row=orig.ws_row, ws_col=orig.ws_col,
+                         ws_xpixel=50000, ws_ypixel=0)  # Too large, zero
+
+        term._height_and_width = mock_height_and_width
+
+        # Should skip invalid TIOCSWINSZ and fall through to XTSMGRAPHICS
+        term.ungetch('\x1b[?2;0;800;600S')  # XTSMGRAPHICS response
+        result = term.get_sixel_height_and_width(timeout=0.1, force=True)
+        assert result == (600, 800)
+        return b'OK'
+
+    output = pty_test(child, parent_func=None,
+                      test_name='test_tiocswinsz_invalid_dimensions')
+    assert 'OK' in output
+
+
+def test_get_sixel_height_and_width_not_a_tty():
+    """Test get_sixel_height_and_width returns (-1, -1) when not a TTY."""
+    @as_subprocess
+    def child():
+        term = TestTerminal(stream=io.StringIO(), force_styling=True)
+        term._is_a_tty = False
+
+        result = term.get_sixel_height_and_width(timeout=0.1)
+        assert result == (-1, -1)
+    child()
